function [instrumentSays] = sendSCPIcmd(visaResource, cmdToSend, withOPC, withReset)
% sendSCPIcmd Directly sends a SCPI string to an instrument resource.
%
% Queries are detected, their collective responses are collated,
% and appended to a string, which is returned as a function output.
%
% Example: sendSCPIcmd('TCPIP::10.85.0.167::INSTR', ':SOURce1:FREQuency:CW 3600000000', 1, 0)
% Without an initial reset, instructs the signal generator with IP address
% 10.85.0.167, to change its center frequency to 3.6GHz
%
% (C) Rohde & Schwarz 2018

%% detect if the resource is already a connected... if not, create

    if (ischar(visaResource))
        objInstr = VISA_Instrument(visaResource);
        openClose = true;
    elseif (isa(visaResource, 'VISA_Instrument'))
        objInstr = visaResource;
        openClose = false;
    else
        error('"visaResource" parameter must be either ResourceName string or "VISA_Instrument" object');
    end

%% prepare the reply string
    instrumentSays = [];

%% reset, if requested
    if withReset
        objInstr.Write('*RST'); objInstr.QueryString('*OPC?');
    end

%% prepare and write the command/query
tline = cmdToSend;

    if strfind(tline,'?')
        instrumentSays = strcat(instrumentSays, ',', objInstr.QueryString(tline));
    else    
        objInstr.Write(tline);
        if withOPC
        objInstr.QueryString('*OPC?');
        end
    end
    
%% if the resource was not passed already opened, then close it 
    if (openClose)
        objInstr.Close();
    end

%% pre-process the instrument response
    if length(instrumentSays)>1
        instrumentSays = strrep(instrumentSays, 'NAN', 'nan');
        instrumentSays = strrep(instrumentSays(2:end),';',',');
        if isnumeric(instrumentSays)
            instrumentSays = str2num(instrumentSays);
        end
    end

end